PROGRAM ImageJ;

//
//  Author: George H. Silva <George.Silva@chemie.bio.uni-giessen.de>
//  Version History:
//    v1.0.0.x - 2006/1/21 - initial release
//             - Basic code based on "The Java launcher for Windows v1.22" by Jacob Marner
//             - Improved/customized for ImageJ (i.e. can handle file-associations, etc)
//             - This windows stub can be used for other Java applications as well;
//               just change the icon, set the DEF_CFG constant and re-compile!
//
//    v1.1.0.x - 2006/1/23 - many updates/improvements
//             - Robust loader to handle various config scenarios
//             - Code to auto-create the config file
//             - Code for auto-detection of various parameters (JVM, mem usage, etc)
//             - Code to allow selection of needed files
//             - This version essentially can reconfigure itself as needed, depending
//               on how the user changes their JAVA install
//

USES
  Windows, SysUtils, ShellAPI, Shared;

{$R *.res}

VAR
  bSaveCfg: Boolean;
  sCfg,sIJDir,sJVM,sParams,sParamsEx: String;

BEGIN
  // Read/Write mode for files (global)
  FileMode:=2;
  bSaveCfg:=false;
  // Look for the config file - default to DEF_CFG or create one as needed
  if not(FileExists(Evalset(ChangeFileExt(ParamStr(0),CFG_EXT),sCfg))) then
    begin
      if SameText(ExtractFileName(sCfg),DEF_CFG) then
        sCfg:=CreateCfgFile('','','','',true) // Create a new CFG file
      else // Try default CFG
        if not(FileExists(EvalSet(ExtractFilePath(ParamStr(0))+DEF_CFG,sCfg))) then
          sCfg:=CreateCfgFile('','','','',true) // Create a new CFG file
    end;
  // Read config and auto-repair if needed
  if not(ReadCfgFile(sCfg,sIJDir,sJVM,sParams)) then
    begin
      if (MessageBox(0,PChar('The configuration file is corrupt. Would you like to '+
                     'auto-repair this file?'),PChar('Bad Configuration File Format'),
                     MB_YESNO or MB_ICONQUESTION) = ID_NO) then
        FatalError('The configuration file should contain at least 3 lines:'+#13#13+
                   'Line 1 - The '+TXT_APP+' install directory (location of JAR file)'+#13+
                   'Line 2 - The location of the java virtual machine (JVM) executable'+#13+
                   'Line 3 - Parameters to pass to the JVM'+#13#13+
                   'To restore the default configuration, simply re-start '+TXT_APP+' after '+
                   'deleting this file:'+#13#13+sCfg,
                   'Bad Configuration File Format',2)
      else
        begin
          sCfg:=CreateCfgFile(sCfg,'','','');
          if not(ReadCfgFile(sCfg,sIJDir,sJVM,sParams)) then
            FatalError('The configuration file could not be repaired. Please check the '+
                       TXT_APP+' installation and try again','Bad Configuration File Format',3)
        end;
    end;
  // Validate the ImageJ folder
  if not(IJJarExists(sIJDir)) then
    begin
      bSaveCfg:=true; // Save new data below if all is ok
      if not(GetValidIJPath(sIJDir,false)) then
        FatalError('Could not locate the '+TXT_APP+' JAR file ('+JAR_IMAGEJ+'). Please '+
                   'check the installation and try again.','Fatal Error',4);
    end;
  // Validate the JVM
  if not(FileExists(sJVM)) then
    begin
      bSaveCfg:=true; // Save new data below if all is ok
      if not(GetValidJVM(sIJDir,sJVM)) then
        FatalError('Could not locate the JVM file ('+EXE_JVM+'). Please '+
                   'check the installation and try again.','Fatal Error',5);
    end;
  // Change to selected directory
  if not(SetCurrentDir(sIJDir)) then
    FatalError('Problem setting the current directory to: "'+sIJDir+'"','Directory Error',6);
  // Get expanded (passed) parameters
  sParamsEx:=Trim(sParams+' '+PassedParams);
  // Run the program
  if (ShellExecute(0,nil,PChar(sJVM),PChar(sParamsEx),PChar(sIJDir),SW_SHOWNORMAL) <=32) then
    FatalError('Could not launch '+TXT_APP+' as configured. Please check the '+
               'settings and try again.','Fatal Error',7)
  else
    if bSaveCfg then // Save this if needed
      CreateCfgFile(sCfg,sIJDir,sJVM,sParams);
END.

